#!/usr/bin/python
# -*- coding: utf-8 -*-
################################################################################
#    Eupompos Copyright (C) 2012 Xavier Faure
#    Contact: suizozukan arrobas orange dot fr
#
#    This file is part of Eupompos.
#    Eupompos is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Eupompos is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Eupompos.  If not, see <http://www.gnu.org/licenses/>.
################################################################################
"""
        ❏Eupompos❏ eupompos/cppparser/liaf.py
        ________________________________________________________________________

        Tools usefull to read/write informations about a line in a file.
        LIAF is a language agnostic class.
        ________________________________________________________________________

        o LIAF class
"""

################################################################################
class LIAF(object):
    """
        L[ine] I[n] A F[ile]
        ________________________________________________________________________

        Simple class to store a filename, the line number and the line itself.
        ________________________________________________________________________

        class attribute :

            o LIAF.max_length_for_filenames : (int) see pos_*() functions.

        instance attributes :

            o filename     : str or None if undefined
            o linenumber   : int or None if undefined
            o line         : str or None if undefined

        class methods :

            o __init__()
            o _repr__()
            o pos_and_line_repr()
            o pos_repr()
            o shorten()             [static]
    """

    # constant used by LIAF.shorten()
    max_length_for_filenames = 100

    #///////////////////////////////////////////////////////////////////////////
    def __init__(self,
                 _filename=None,
                 _linenumber=None,
                 _line=None):
        """
                LIAF.__init__()
                ________________________________________________________________

                PARAMETERS :

                o _filename     : (str, None if undefined)
                o _linenumber   : (int, None if undefined)
                o _line         : (str, None if undefined)

                RETURN VALUE    : no return value
        """
        self.filename = _filename
        self.linenumber = _linenumber
        self.line = _line

    #///////////////////////////////////////////////////////////////////////////
    def __repr__(self):
        """
                LIAF.__repr__()
                ________________________________________________________________

                Return the state of the object in human readable way.
                All the informations appear in the returned string.
                ________________________________________________________________

                PARAMETER(S)    : no parameter

                RETURN VALUE    : a string
        """
        return self.pos_and_lines_repr()

    #///////////////////////////////////////////////////////////////////////////
    def pos_and_lines_repr(self, _max_length_for_filenames=max_length_for_filenames):
        """
                LIAF.pos_and_lines_repr()
                ________________________________________________________________

                Return the state of the object in human readable way.
                All the informations appear in the returned string.
                ________________________________________________________________

                PARAMETER(S)    : no parameter

                RETURN VALUE    : a string
        """
        return "{0}#{1}\n{2}\"{3}\"".format(LIAF.shorten(self.filename, _max_length_for_filenames),
                                            self.linenumber,
                                            " "*3,
                                            self.line)

    #///////////////////////////////////////////////////////////////////////////
    def pos_repr(self, _max_length_for_filenames=max_length_for_filenames):
        """
                LIAF.pos_repr()
                ________________________________________________________________

                Return the state of the object in human readable way.

                The line's content is not displayed.
                ________________________________________________________________

                PARAMETER(S)    : no parameter

                RETURN VALUE    : a string
        """
        return "{0}#{1}".format(LIAF.shorten(self.filename, _max_length_for_filenames),
                                self.linenumber)

    #///////////////////////////////////////////////////////////////////////////
    @staticmethod
    def shorten(_string, _max_length_for_filenames):
        """
                LIAF.shorten()
                ________________________________________________________________

                Return a shortened version of "string", up to
                _max_length_for_filenames characters.
                ________________________________________________________________

                PARAMETER :
                o _string       : (str) the source string

                RETURN VALUE    : a string
        """
        if len(_string) > _max_length_for_filenames:
            return "(...)" + _string[len(_string) - _max_length_for_filenames:]
        else:
            return _string
