#!/usr/bin/python
# -*- coding: utf-8 -*-
################################################################################
#    Eupompos Copyright (C) 2012 Xavier Faure
#    Contact: suizozukan arrobas orange dot fr
#
#    This file is part of Eupompos.
#    Eupompos is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Eupompos is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Eupompos.  If not, see <http://www.gnu.org/licenses/>.
################################################################################
"""
        ❏Eupompos❏ eupompos/pyparserstate.py

        o PyParserState class
"""
# something's wrong with the way pylint understands the import statement :
# pylint: disable=import-error

from eupompos.parsertools import FunctionName

################################################################################
class PyParserState(object):
    """
        PyParserState class
        ________________________________________________________________________

        Class used by a PyParser object to store temporary informations about
        the current file.
        ________________________________________________________________________

        class attributes : -

        instance attributes :

            o current_block_of_comments : list of strings
            o current_class : str
            o current_connections_line : list of strings
            o current_function : FunctionName

        methods :

            o __init__()
            o __repr__()
            o reset()
            o xrepr()
    """

    #//////////////////////////////////////////////////////////////////////////
    def __init__(self):
        """
                PyParserState.__init__()
                ________________________________________________________________

                Create the attributes, set them to None and call .reset() to
                initialize these attributes to a default value.
                ________________________________________________________________

                PARAMETER(S) : no parameter

                RETURN VALUE : no return value
        """
        self.current_block_of_comments = None
        self.current_class = None
        self.current_connections_line = None
        self.current_function = None

        self.reset()

    #//////////////////////////////////////////////////////////////////////////
    def __repr__(self):
        """
                PyParserState.__repr__()
                ________________________________________________________________

                Return a string describing self.
                ________________________________________________________________

                PARAMETER(S) : no parameter

                RETURN VALUE : no return value
        """
        return self.xrepr()

    #//////////////////////////////////////////////////////////////////////////
    def reset(self):
        """
                PyParserState.reset()
                ________________________________________________________________

                Set the attributes to their default value.
                ________________________________________________________________

                PARAMETER(S) : no parameter

                RETURN VALUE : no return value
        """
        self.current_block_of_comments = []
        self.current_class = None
        self.current_connections_line = []
        self.current_function = FunctionName(None, "")

    #//////////////////////////////////////////////////////////////////////////
    def xrepr(self):
        """
                PyParserState.xrepr()
                ________________________________________________________________

                Return a string describing self.
                ________________________________________________________________

                PARAMETER(S) : no parameter

                RETURN VALUE : no return value
        """
        return "(PyParserState) class={0}; " \
               "connections line={1}; " \
               "function={2}; comments={3}".format(self.current_class,
                                                   self.current_connections_line,
                                                   self.current_function,
                                                   self.current_block_of_comments)
